import java.lang.*;

/*
-------------------------------------------------------------------------------
Equacions de Lotka Volterra (sense normalitzar). Inclou versio discreta i
continua
-------------------------------------------------------------------------------
Dades: dt (duracio d'un pas de temps), imax (nombre de passos), jmax
       (iteracions per resoldre les eqs. diferencials), nx0 (nombre inicial
       de membres especie X), ny0 (nombre inicial de membres especie Y),
       a (parametre alfa), b (parametre beta), Rx (factor de creixement
       especie X), Ry (factor de creixement especie Y), Kx (capacitat especie
       X), Ky (capacitat especie Y).
Resultats: Fitxer "herba2.dat" estructurat en 5 columnes. Cada fila representa
           un instant de temps. La primera columna es el temps transcorregut.
           La segona i la tercera son el nombre de membres de les especies
           X i Y, respectivament, segons el model discret. La quarta i la
           cinquena son tambe el nombre de membres de les especies X i Y,
           pero segons el model continu.
Dades de prova: dt=0.1 imax=2 jmax=1 nx0=300 ny0=50 a=0.1 b=0.1 Rx=2.0
                Ry=2.0 Kx=400 Ky=400 ==> 0.0, 300.0, 50.0, 300.0, 50.0;
                0.1, 349.38674, 77.725887, 371.25001, 89.999998;
                0.2, 375.324249, 116.426730, 389.580464, 151.396871
Observacions: Tots els parametres corresponen al model sense normalitzar!
              Si no compila be, prova amb l'opcio -lm
-------------------------------------------------------------------------------
*/

public class Herba2Calculs {
	double dt,nx0,ny0,a,b,Rx,Ry,Kx,Ky,temps[],trajx[],trajy[],trajxc[],trajyc[];
	int imax;

	Herba2Calculs(float dt0,int imax0,float nx00,float ny00,float a0,float b0,float Rx0,float Ry0,float Kx0,float Ky0) {
		double dx,dy,x0,y0,t0,x,y,t,xf,yf,tf,Xf,Yf,xm,ym,tm,Qx,Qy,rx,ry,X,Y;
		int i,j,jmax;

		dt = (double) dt0;
		nx0 = (double) nx00;
		ny0 = (double) ny00;
		a = (double) a0;
		b = (double) b0;
		Rx = (double) Rx0;
		Ry = (double) Ry0;
		Kx = (double) Kx0;
		Ky = (double) Ky0;
		imax = imax0;
		jmax = 1;
		temps = new double[imax];
		trajx = new double[imax];
		trajy = new double[imax];
		trajxc = new double[imax];
		trajyc = new double[imax];

/* Hipotesi 1 */

		rx=Math.log(Rx)/dt;
		ry=Math.log(Ry)/dt;

/* Hipotesi 2 */

		Qx=Kx*((1.0/(Rx-1))+1.0);
		Qy=Ky*((1.0/(Ry-1))+1.0);

/* Normalitzacio */

		x0=nx0/Kx ;
		y0=ny0/Ky ;
		X=nx0/Qx ;
		Y=ny0/Qy ;

		t0=0;
		x=x0;
		y=y0;
		t=t0;
		xf=x0;
		yf=y0;
		temps[0]=t0;
		trajx[0]=x0*Kx;
		trajy[0]=y0*Ky;
		trajxc[0]=X*Qx;
		trajyc[0]=Y*Qy;

		for(i=0;i<imax;i++) {
			tm=t+dt/2.0;
			for(j=0;j<jmax;j++) {
				xm=(x+xf)/2.0;
				ym=(y+yf)/2.0;
				dx=rx*(1.0-xm-a*Qx/Kx*Ky/Qy*ym)*xm*dt;
				dy=ry*(1.0-ym-b*Qy/Ky*Kx/Qx*xm)*ym*dt;
				xf=x+dx;
				yf=y+dy;
			}
			x=xf;
			y=yf;
			t=t+dt;
			Xf=Rx*(1.0-X-a*Y)*X;
			Yf=Ry*(1.0-Y-b*X)*Y;
			X=Xf;
			Y=Yf;
			temps[i]=t;
			trajx[i]=x*Kx;
			trajy[i]=y*Ky;
			trajxc[i]=X*Qx;
			trajyc[i]=Y*Qy;
		}
	}
}
